==> sqlitedbms-0.5.0.tar.gz: sqlitedbms-0.5.0/src/backend/log.c <==
#include "include/common.h"

struct log_file_t	*log_file;
struct log_file_t	*log_file_last;

struct log_entry_t	*log_entry;
struct log_entry_t	*log_entry_last;
pthread_t		log_thread;
pthread_mutex_t		log_lock;

void log_init()
{
	log_entry = log_entry_last = NULL;
	log_file = log_file_last = NULL;
	pthread_mutex_init( &log_lock, NULL );

	pthread_create( &log_thread, NULL, log_proc, NULL );

	return;
}

void *log_proc( void *nil )
{
	int			date;
	struct log_file_t	*file;
	struct log_entry_t	*tmp;

	date = log_now();
	while (1)
	{
		while ( log_entry != NULL )
		{
			//printf("%p, %p\n", log_entry, log_entry->next);
			file = log_get_file( log_entry, date );
			if ( file != NULL )
			{
				fwrite( log_entry->msg, strlen(log_entry->msg), 1, file->file );
				fflush( file->file );
			}

			tmp = log_entry;
			log_entry = log_entry->next;
			pthread_mutex_lock( &log_lock );

			if ( log_entry == NULL )
				log_entry_last = NULL;
			pthread_mutex_unlock( &log_lock );
			log_entry_free( tmp );

		}
		sleep(1);
		date = log_now();

		log_analyze( date );
		//ϱ, flush
	}

	pthread_exit(0);
}

struct log_file_t *log_get_file( struct log_entry_t *entry, int date )
{
	extern struct config_t	config;

	struct log_file_t	*file;
	size_t			filename_len;

	for ( file=log_file; file; file=file->next )
	{
		if ( entry->type == file->type && file->date == date )
		{
			if ( entry->type == LOG_TYPE_DB )
			{
				if ( strcmp( entry->dbname, file->dbname ) == 0 )
					return file;
			}
			else
				return file;
		}
	}

	file = malloc( sizeof(struct log_file_t) );
	if ( file == NULL )
		return NULL;

	memset( file, 0, sizeof(struct log_file_t) );
	file->type = entry->type;
	file->date = date;

	filename_len = strlen( config.log_filerule ) + 128;

	if ( entry->type == LOG_TYPE_DB )
	{
		file->dbname = malloc( strlen( entry->dbname ) + 1 );
		strcpy( file->dbname, entry->dbname );

		filename_len += strlen( entry->dbname );
	}

	file->filename = malloc( filename_len );

	switch ( entry->type )
	{
		case LOG_TYPE_SERVER:
			log_make_filename( file->filename, config.log_filerule, "server", date );
		break;
		case LOG_TYPE_DB:
			log_make_filename( file->filename, config.log_filerule, file->dbname, date );
		break;
		case LOG_TYPE_HTTP:
			log_make_filename( file->filename, config.log_filerule, "http", date );
		break;
	}

	file->file = fopen( file->filename, "a" );
	if ( file->file == NULL )
	{
		perror("Log file open error");
		free( file->dbname );
		free( file );
		return NULL;
	}

	file->prev = log_file_last;
	if ( log_file_last == NULL )
		log_file = log_file_last = file;
	else
	{
		log_file_last->next = file;
		log_file_last = file;
	}

	return file;
}

char *log_make_filename( char *dst, char *format, char *type, int date )
{
	extern struct config_t	config;

	dst += sprintf( dst, "%s/", config.log_path );

	while ( (format[0]) != '\0' )
	{
		if ( format[0] == '%' )
		{
			if ( strncmp( "%DATE", format, 5 ) == 0 )
			{
				format += 4;
				dst += sprintf( dst, "%d", date );
			}
			else if ( strncmp( "%TYPE", format, 5 ) == 0 )
			{
				format += 4;
				dst += sprintf( dst, "%s", type );
			}
			else
			{
				dst[0] = format[0];
				dst++;
			}
		}
		else
		{
			dst[0] = format[0];
			dst++;
		}
		format++;
	}
	dst[0] = '\0';

	return dst;
}

void log_analyze( int date )
{
	struct log_file_t	*file;
	struct log_file_t	*i;

	for ( i=file=log_file; file;)
	{
		i = file->next;

		if ( file->date < date )
		{
			if ( file->prev == NULL )
				log_file = file->next;
			else
				file->prev->next = file->next;

			if ( file->next == NULL )
				log_file_last = file->prev;
			else
				file->next->prev = file->prev;

			fclose( file->file );
			if ( file->dbname != NULL )
				free( file->dbname );
			if ( file->filename != NULL )
				free( file->filename );
			free( file );
		}
		else
			fflush( file->file );

		file = i;
	}
}

void log_append( short type, char *dbname, char *msg, short allocated )
{
	struct log_entry_t	*new_log;

	new_log = malloc( sizeof(struct log_entry_t) );
	memset( new_log, 0, sizeof(struct log_entry_t) );

	new_log->type = type;
	new_log->dbname = dbname;
	new_log->msg = msg;
	new_log->allocated = allocated;

	pthread_mutex_lock( &log_lock );
	if ( log_entry_last == NULL )
		log_entry = log_entry_last = new_log;
	else
	{
		log_entry_last->next = new_log;
		log_entry_last = new_log;
	}
	pthread_mutex_unlock( &log_lock );
}

void log_server( const char *msg )
{
	size_t	len_buf;
	char	*buf;
	char	now[128];

	len_buf = strlen( msg ) + 128;

	buf = malloc( len_buf );
	snprintf( buf, len_buf, "[%s] %s\n", log_get_time( now ), msg );

	log_append( LOG_TYPE_SERVER, NULL, buf, LOG_ALLOC_MSG );

	return;
}

void log_server_print( const char *msg, ... )
{
	va_list	argv;
	size_t	len;
	char	*buf;
	char	*tmp;
	char	now[128];

	buf = malloc( 1024 );
	tmp = buf;
	len = sprintf( buf, "[%s] ", log_get_time( now ) );
	tmp += len;
	va_start( argv, msg );
	tmp += vsnprintf( tmp, 1024-len, msg, argv );
	va_end( argv );
	tmp[0] = '\n';
	tmp[1] = '\0';

	log_append( LOG_TYPE_SERVER, NULL, buf, LOG_ALLOC_MSG );

	return;
}

void log_db( char *ip, short method, char *id, char *dbname, char *sql, short status, int sended )
{
	size_t	len_dbname;
	char	*buf_dbname;
	size_t	len_buf;
	char	*buf;
	char	now[128];

	len_dbname = strlen( dbname );
	if ( len_dbname <= 0 )
		return;

	buf_dbname = malloc( len_dbname + 1 );
	strcpy( buf_dbname, dbname );

	len_buf = strlen( sql ) + len_dbname + 256;

	buf = malloc( len_buf );
	snprintf( buf, len_buf, "%s %s [%s] %d %d\n%s\n\n", ip, id?id:"-", log_get_time( now ), status, sended, sql );

	log_append( LOG_TYPE_DB, buf_dbname, buf, LOG_ALLOC_DBNAME|LOG_ALLOC_MSG );

	return;
}

void log_http( char *ip, short method, char *id, char *uri, short status, int sended )
{
	size_t	len_buf;
	char	*buf;
	char	now[128];

	len_buf = strlen( uri ) + 256;

	buf = malloc( len_buf );
	snprintf( buf, len_buf, "%s %s [%s] \"%s /%s\" %d %d\n", ip, id?id:"-", log_get_time( now ), log_http_method( method ), uri, status, sended );

	log_append( LOG_TYPE_HTTP, NULL, buf, LOG_ALLOC_MSG );

	return;
}

void log_entry_free( struct log_entry_t *entry )
{
	//printf("%s, %s\n", entry->dbname , entry->msg);
	//printf("%p, %p, %p\n", entry->dbname , entry->msg, entry);
	if ( entry->allocated & LOG_ALLOC_DBNAME )
		free( entry->dbname );
	if ( entry->allocated & LOG_ALLOC_MSG )
		free( entry->msg );
	free( entry );
}

int log_now()
{
	time_t		now;
	struct tm	tm;

	time( &now );
	localtime_r( &now, &tm );

	return ((1900+tm.tm_year)*10000)+((tm.tm_mon+1)*100)+tm.tm_mday;
}

char *log_get_time( char *buf )
{
	time_t		now;
	struct tm	tm;

	time( &now );
	localtime_r( &now, &tm );

	strftime( buf, 128, "%d/%b/%Y:%T %z", &tm );

	return buf;
}

char *log_http_method( short method )
{
	switch ( method )
	{
		case REQUEST_METHOD_POST:
			return "POST";
		break;
		case REQUEST_METHOD_GET:
			return "GET";
		break;
		case REQUEST_METHOD_HEAD:
			return "HEAD";
		break;
		case REQUEST_METHOD_OPTION:
			return "OPTION";
		break;
		case REQUEST_METHOD_PUT:
			return "PUT";
		break;
		case REQUEST_METHOD_DELETE:
			return "DELETE";
		break;
	}
	return "-";
}

void log_safe_exit()
{
	struct log_entry_t	*tmp;

	pthread_mutex_lock( &log_lock );
	while ( log_entry != NULL )
	{
		tmp = log_entry;
		log_entry = log_entry->next;
		log_entry_free( tmp );
	}
	pthread_mutex_unlock( &log_lock );
	pthread_mutex_destroy( &log_lock );

	log_analyze( log_now() + 999 );
}

==> sqlitedbms-0.5.1.tar.gz: sqlitedbms-0.5.1/src/backend/log.c <==
#include "include/common.h"

struct log_file_t	*log_file;
struct log_file_t	*log_file_last;

short			log_able = FALSE;

struct log_entry_t	*log_entry;
struct log_entry_t	*log_entry_last;
pthread_t		log_thread;
pthread_mutex_t		log_lock;

int log_init()
{
	extern struct config_t  config;
	FILE                    *test;
	char                    filename[512];

	snprintf( filename, 512, "%s/.init", config.log_path ); 
	test = fopen( filename, "w" );
	if ( test == NULL )
		return SQLITE_ERROR;
	fclose( test );
	unlink( filename );

	log_entry = log_entry_last = NULL;
	log_file = log_file_last = NULL;
	pthread_mutex_init( &log_lock, NULL );

	log_able = TRUE;
	pthread_create( &log_thread, NULL, log_proc, NULL );

	return SQLITE_OK;
}

void *log_proc( void *nil )
{
	int			date;
	struct log_file_t	*file;
	struct log_entry_t	*tmp;

	date = log_now();
	while (1)
	{
		if ( log_able == FALSE )
			break;

		while ( log_entry != NULL )
		{
			//printf("%p, %p\n", log_entry, log_entry->next);
			file = log_get_file( log_entry, date );
			if ( file != NULL )
			{
				fwrite( log_entry->msg, strlen(log_entry->msg), 1, file->file );
				fflush( file->file );
			}

			tmp = log_entry;
			log_entry = log_entry->next;
			pthread_mutex_lock( &log_lock );

			if ( log_entry == NULL )
				log_entry_last = NULL;
			pthread_mutex_unlock( &log_lock );
			log_entry_free( tmp );

		}
		sleep(1);
		date = log_now();

		//reorg, flush
		log_analyze( date );
	}

	log_safe_exit();
	pthread_exit(0);
}

struct log_file_t *log_get_file( struct log_entry_t *entry, int date )
{
	extern struct config_t	config;

	struct log_file_t	*file;
	size_t			filename_len;

	for ( file=log_file; file; file=file->next )
	{
		if ( entry->type == file->type && file->date == date )
		{
			if ( entry->type == LOG_TYPE_DB )
			{
				if ( strcmp( entry->dbname, file->dbname ) == 0 )
					return file;
			}
			else
				return file;
		}
	}

	file = malloc( sizeof(struct log_file_t) );
	if ( file == NULL )
		return NULL;

	memset( file, 0, sizeof(struct log_file_t) );
	file->type = entry->type;
	file->date = date;

	filename_len = strlen( config.log_filerule ) + 128;

	if ( entry->type == LOG_TYPE_DB )
	{
		file->dbname = malloc( strlen( entry->dbname ) + 1 );
		strcpy( file->dbname, entry->dbname );

		filename_len += strlen( entry->dbname )+1;
	}

	file->filename = malloc( filename_len );

	switch ( entry->type )
	{
		case LOG_TYPE_SERVER:
			log_make_filename( file->filename, config.log_filerule, "server", date );
		break;
		case LOG_TYPE_DB:
			log_make_filename( file->filename, config.log_filerule, file->dbname, date );
		break;
		case LOG_TYPE_HTTP:
			log_make_filename( file->filename, config.log_filerule, "http", date );
		break;
	}

	file->file = fopen( file->filename, "a" );
	if ( file->file == NULL )
	{
		perror("Log file open error");
		if ( file->dbname != NULL )
			free( file->dbname );
		free( file->filename );
		free( file );
		return NULL;
	}

	free( file->filename );
	file->filename = NULL;

	file->prev = log_file_last;
	if ( log_file_last == NULL )
		log_file = log_file_last = file;
	else
	{
		log_file_last->next = file;
		log_file_last = file;
	}

	return file;
}

char *log_make_filename( char *dst, char *format, char *type, int date )
{
	extern struct config_t	config;

	dst += sprintf( dst, "%s/", config.log_path );

	while ( (format[0]) != '\0' )
	{
		if ( format[0] == '%' )
		{
			if ( strncmp( "%DATE", format, 5 ) == 0 )
			{
				format += 4;
				dst += sprintf( dst, "%d", date );
			}
			else if ( strncmp( "%TYPE", format, 5 ) == 0 )
			{
				format += 4;
				dst += sprintf( dst, "%s", type );
			}
			else
			{
				dst[0] = format[0];
				dst++;
			}
		}
		else
		{
			dst[0] = format[0];
			dst++;
		}
		format++;
	}
	dst[0] = '\0';

	return dst;
}

void log_analyze( int date )
{
	struct log_file_t	*file;
	struct log_file_t	*i;

	for ( i=file=log_file; file;)
	{
		i = file->next;

		if ( file->date < date )
		{
			if ( file->prev == NULL )
				log_file = file->next;
			else
				file->prev->next = file->next;

			if ( file->next == NULL )
				log_file_last = file->prev;
			else
				file->next->prev = file->prev;

			fclose( file->file );
			if ( file->dbname != NULL )
				free( file->dbname );
			free( file );
		}
		else
			fflush( file->file );

		file = i;
	}
}

void log_append( short type, char *dbname, char *msg, short allocated )
{
	struct log_entry_t	*new_log;

	new_log = malloc( sizeof(struct log_entry_t) );
	memset( new_log, 0, sizeof(struct log_entry_t) );

	new_log->type = type;
	new_log->dbname = dbname;
	new_log->msg = msg;
	new_log->allocated = allocated;

	pthread_mutex_lock( &log_lock );
	if ( log_entry_last == NULL )
		log_entry = log_entry_last = new_log;
	else
	{
		log_entry_last->next = new_log;
		log_entry_last = new_log;
	}
	pthread_mutex_unlock( &log_lock );
}

void log_server( const char *msg )
{
	size_t	len_buf;
	char	*buf;
	char	now[128];

	if ( log_able == FALSE )
		return;

	len_buf = strlen( msg ) + 128;

	buf = malloc( len_buf );
	snprintf( buf, len_buf, "[%s] %s\n", log_get_time( now ), msg );

	log_append( LOG_TYPE_SERVER, NULL, buf, LOG_ALLOC_MSG );

	return;
}

void log_server_print( const char *msg, ... )
{
	va_list	argv;
	size_t	len;
	char	*buf;
	char	*tmp;
	char	now[128];

	if ( log_able == FALSE )
		return;

	buf = malloc( 1024 );
	tmp = buf;
	len = sprintf( buf, "[%s] ", log_get_time( now ) );
	tmp += len;
	va_start( argv, msg );
	tmp += vsnprintf( tmp, 1024-len, msg, argv );
	va_end( argv );
	tmp[0] = '\n';
	tmp[1] = '\0';

	log_append( LOG_TYPE_SERVER, NULL, buf, LOG_ALLOC_MSG );

	return;
}

void log_db( char *ip, short method, char *id, char *dbname, char *sql, short status, int sended )
{
	size_t	len_dbname;
	char	*buf_dbname;
	size_t	len_buf;
	char	*buf;
	char	now[128];

	if ( log_able == FALSE )
		return;

	len_dbname = strlen( dbname );
	if ( len_dbname <= 0 )
		return;

	buf_dbname = malloc( len_dbname + 1 );
	strcpy( buf_dbname, dbname );

	len_buf = strlen( sql ) + len_dbname + 256;

	buf = malloc( len_buf );
	snprintf( buf, len_buf, "%s %s [%s] %d %d\n%s\n\n", ip, id?id:"-", log_get_time( now ), status, sended, sql );

	log_append( LOG_TYPE_DB, buf_dbname, buf, LOG_ALLOC_DBNAME|LOG_ALLOC_MSG );

	return;
}

void log_http( char *ip, short method, char *id, char *uri, short status, int sended )
{
	size_t	len_buf;
	char	*buf;
	char	now[128];

	if ( log_able == FALSE )
		return;

	len_buf = strlen( uri ) + 256;

	buf = malloc( len_buf );
	snprintf( buf, len_buf, "%s %s [%s] \"%s /%s\" %d %d\n", ip, id?id:"-", log_get_time( now ), log_http_method( method ), uri, status, sended );

	log_append( LOG_TYPE_HTTP, NULL, buf, LOG_ALLOC_MSG );

	return;
}

void log_entry_free( struct log_entry_t *entry )
{
	//printf("%s, %s\n", entry->dbname , entry->msg);
	//printf("%p, %p, %p\n", entry->dbname , entry->msg, entry);
	if ( entry->allocated & LOG_ALLOC_DBNAME )
		free( entry->dbname );
	if ( entry->allocated & LOG_ALLOC_MSG )
		free( entry->msg );
	free( entry );
}

int log_now()
{
	time_t		now;
	struct tm	tm;

	time( &now );
	localtime_r( &now, &tm );

	return ((1900+tm.tm_year)*10000)+((tm.tm_mon+1)*100)+tm.tm_mday;
}

char *log_get_time( char *buf )
{
	time_t		now;
	struct tm	tm;

	time( &now );
	localtime_r( &now, &tm );

	strftime( buf, 128, "%d/%b/%Y:%T %z", &tm );

	return buf;
}

char *log_http_method( short method )
{
	switch ( method )
	{
		case REQUEST_METHOD_POST:
			return "POST";
		break;
		case REQUEST_METHOD_GET:
			return "GET";
		break;
		case REQUEST_METHOD_HEAD:
			return "HEAD";
		break;
		case REQUEST_METHOD_OPTION:
			return "OPTION";
		break;
		case REQUEST_METHOD_PUT:
			return "PUT";
		break;
		case REQUEST_METHOD_DELETE:
			return "DELETE";
		break;
	}
	return "-";
}

void log_safe_exit()
{
	struct log_entry_t	*tmp;

	pthread_mutex_lock( &log_lock );
	while ( log_entry != NULL )
	{
		tmp = log_entry;
		log_entry = log_entry->next;
		log_entry_free( tmp );
	}
	pthread_mutex_unlock( &log_lock );
	pthread_mutex_destroy( &log_lock );

	log_analyze( log_now() + 999 );
}
